function [out,t,RMSE] = SBMIR(o,u0,op,z1,dz,lambda,deltaA,deltaB,ITER)
% SBMIR() realizes the succesive phase-retrieval algorithm called "single-beam
% multiple-intensity phase reconstruction" (SBMIR) focused on the sesnor planes: 
% the reconstruction of a volumetric sensor plane wave field or a complex-valued 
% object as it is described in [1]. The diffraction wave field 
% propagation is performed by the angular spectrum decomposition (ASD).
%
% Notation:
% [out,t,RMSE] = SBMIR(o,u0,op,z1,dz,lambda,deltaA,deltaB,ITER,q)
% ------------------------------- input -----------------------------------
% o                     3D matrix of the (noisy) intensity observations of the size NxM 
%                       for K sensor planes, i.e. the size of the matrix must be NxMxK,
%                       where N, M and K are positive even integers, K>1
% u0                    true object of the size NxM or smaller used to calculate the 
%                       reconstruction accuracy (and algorithm convergence) in RMSE
% op                    this parameter determines what method of SBMIR to use:
%                       If op = 0 - forward-backward (FB) model (from the first to the K-th plane 
%                       and then back to th first plane)
%                       If op = 1 - backward-forward (BF) model (from the K-th to the first plane 
%                       and then back to the K-th plane)
%                       If op = 2 - forward-forward (FF) model (from the first to the K-th plane 
%                       and then back to the fist plane)
% z1                    distance between the object and sensor plane [m]
% dz                    distance between the observation planes [m]
% lambda                wavelength [m]
% deltaA                pitch size w.r.t. rows of the discrete object/sensor plane wave field [m]
% deltaB                pitch size w.r.t. columns of the discrete object/sensor plane wave field [m] 
% ITER                  number of iterations of the SBMIR algorithm (optional, by default ITER=100)
% ------------------------------- output ----------------------------------
% out                   If u0=[] is not defined, the output is the NxMxK complex-valued matrix of the 
%                       reconstructed volumetric wave field in K sesor planes
%                       Otherwise 'out' is the reconstructed object of the size of the input 'u0' 
% t                     the computational time in seconds (optional)
% RMSE                  the reconstruction accuracy (via RMSE) at all ITER iteration of the SBMIR
%                       phase retrieval algorithm w.r.t. the object amplitude (RMSE(1,:)) and object phase (RMSE(2,:))    
%
% Example:
% Reconstruction of a complex-valued object from K=5 noiseless observations via SBMIR
%
% u0 = im2double(imread('Lena256.png')); u0 = u0+0.1;    u0 = u0/max(u0(:));
% [N,M]=size(u0);                             % object size
% Nz = N; Mz = M;                             % sesnor size (be default, the same as the object size)
% lambda = 532e-9;                            % wavelength [m]
% delta = 6.7e-6;                             % square pixel size with 100% fill factor [m]
% ITER = 100;                                 % the number of iterations
% z1 =   0.0432;                              % distance to the first sensor plane
% dz = 2e-3;                                  % distance between measurement planes [m]
% K = 5;                                      % number of measurements
% sigma = 0*ones(K,1);                        % noise level of the intnesity observations
% gamma = 10*ones(K,1);                       % penalty coefficient of the AL base algorithms
% mufactor = 0.0005;                          % regularization parameter
% uz = zeros(Nz,Mz,K); o = zeros(Nz,Mz,K);noise = randn(Nz,Mz,K);
% for index = 1:K
%     uz(:,:,index) = FDDT(u0,0,z1+(index-1)*dz,lambda,delta,delta,[1,1]);
%     o(:,:,index)= abs(uz(:,:,index)).^2 + sigma(index)*noise(:,:,index);
%     o(:,:,index)=o(:,:,index).*(o(:,:,index)>=0) + 0.0001*(o(:,:,index)<0); % positive projection to avoid negative or zero intensity
% end
% op = 0;                                     % forward-forward SBMIR
% [u0est,t,RMSE] = SBMIR(o,u0,op,z1,dz,lambda,delta,delta,ITER);
%
%
% Reference:            [1] P. Almoro, G. Pedrini, and W. Osten, Appl. Opt. 45, 8596-8605 (2006). 
% Authors:              Artem Migukin
% Software version:     2.0, January 2013.
% Required functions:   TransferFunctionASD()

% default parameters (if not specified)
dITER = 100;         % number of iteration of the phase-retrieval algorithm
q = [1,1];           % the redundancy of the computantional size w.r.t. the y- and x-direction, qx>=1 and qy>=1 
                     % (by default ASD is used of the same size as the size of the object/sensor plane wave field dq=[1,1],
                     % for instance, for realization of ASD via double size zero-padding , please set dq = [2,2].)

                     
% --------------- please do not change the code under this line -----------------------------------------
i = sqrt(-1); RMSE = []; t=0;
if (nargin<9 || isempty(ITER)),  ITER=dITER; end

[N,M,K]=size(o);                 % the size of the input set of the (noisy) intensity observations
if mod(N,2)~=0 || mod(M,2)~=0
   disp('Error: The size of the intensity observations must be even integers'); out = []; return 
end
cRMSE = false;                   % control parameter which control whether RMSE should be calculated
                                 % it the true u0 is given - RMSE is computed and the output variable 'out' is the calculated object 
                                 % otherwise - RMSE=[] is empty variable and 'out' is the calculated volumetric sensor plane wave field
if q(1) < 1 || q(2) < 1, disp('Error: q must be not smaller than 1'); out = []; return; end
Nya = ceil(q(1)*N/2)*2;  Nxa = ceil(q(2)*M/2)*2;     % the support of the computed Fourier images
if K<2
    disp('Error: It must be at least 2 intensity observations, otherwise phase retrieval doesnt make sense'); out = []; return 
end
if ~isempty(u0)                  % the input object is used to estimate the reconstruction accuracy in RMSE
    [Ny0,Nx0,tmp] = size(u0);  % the size of the input object
    if Ny0>Nya || Nx0>Nxa
        disp('Error: The size of the object "u0" should not be larger than the size of the intensity observations!');out = []; return;
    end
    if mod(Ny0,2)~=0 || mod(Nx0,2)~=0
        disp('Error: The size of the object "u0" must be even integers'); out = []; return 
    end
    cRMSE = true; RMSE = zeros(2,ITER); figure(5),close(5)
end 
o = sqrt(o); % the amplitude observation
coordy = Nya/2-N/2+1:Nya/2+N/2; coordx = Nxa/2-M/2+1:Nxa/2+M/2; % center part of the sensor plane wave field
bourdery0 = Nya/2-Ny0/2+1:Nya/2+Ny0/2; bourderx0 = Nxa/2-Nx0/2+1:Nxa/2+Nx0/2; % center part of the object
% calculation of the transfer functions for the forward and backward ASD diffraction propagation to the distance dz
if nargout > 1,    tic;end  
Sf = TransferFunctionASD( dz,lambda,deltaA,deltaB,Nya,Nxa);
switch op
    case 0
        Sb = TransferFunctionASD(-dz,lambda,deltaA,deltaB,Nya,Nxa);
        if cRMSE
            S0 = TransferFunctionASD(-z1,lambda,deltaA,deltaB,Nya,Nxa); 
        end
    case 1
        Sb = TransferFunctionASD(-dz,lambda,deltaA,deltaB,Nya,Nxa);
        if cRMSE
            S0 = TransferFunctionASD(-z1-(K-1)*dz,lambda,deltaA,deltaB,Nya,Nxa); 
        end
    case 2
        Sb = TransferFunctionASD(-(K-1)*dz,lambda,deltaA,deltaB,Nya,Nxa);
        if cRMSE
            S0 = TransferFunctionASD(-z1,lambda,deltaA,deltaB,Nya,Nxa); 
        end
    otherwise
        disp('Error: Wrong setting of the operation, please choose op={0,1,2}.'); out=[];t=0; return
end
if nargout > 1,    t=t+toc; end

ep = ones(Nya,Nxa);     % initial guess for the phase (for phase recovering)
uztmp = zeros(Nya,Nxa)+.0001*rand(Nya,Nxa)+.0001*i*rand(Nya,Nxa);
for jndex = 1:ITER
    if nargout > 1,    tic;end                                              % start a stopwatch timer 
    switch op
        case 0 % FB
        for index=1:1:K-1 % forward
            uztmp(coordy,coordx)=o(:,:,index).*ep(coordy,coordx);
            uztmp=ifft2((fft2(uztmp)).*Sf);% ->           2...K
            ph = atan2(imag(uztmp), real(uztmp));          ep = exp(i*ph);    
        end
        for index=K:-1:2 % backward
            uztmp(coordy,coordx)=o(:,:,index).*ep(coordy,coordx);
            uztmp=ifft2((fft2(uztmp)).*Sb); % <-          K-1...1
            ph = atan2(imag(uztmp), real(uztmp));          ep = exp(i*ph);
        end 
        case 1 % BF
        for index=K:-1:2 % backward
            uztmp(coordy,coordx)=o(:,:,index).*ep(coordy,coordx);       
            uztmp=ifft2((fft2(uztmp)).*Sb); % <-          K-1...1
            ph = atan2(imag(uztmp), real(uztmp));          ep = exp(i*ph);
        end
        for index=1:1:K-1 % forward
            uztmp(coordy,coordx)=o(:,:,index).*ep(coordy,coordx);       
            uztmp=ifft2((fft2(uztmp)).*Sf); % ->          2...K 
            ph = atan2(imag(uztmp), real(uztmp));          ep = exp(i*ph);         
        end
        case 2 % FF
        for index=1:1:K
            uztmp(coordy,coordx)=o(:,:,index).*ep(coordy,coordx);
            if index<K
                uztmp=ifft2((fft2(uztmp)).*Sf); % ->      2...K-1
                ph = atan2(imag(uztmp), real(uztmp));      ep = exp(i*ph);
            else
                uztmp=ifft2((fft2(uztmp)).*Sb); % <-      K->1   
                ph = atan2(imag(uztmp), real(uztmp));      ep = exp(i*ph);
            end
        end
        otherwise
        disp('Error: Wrong setting of the operation, please choose op={0,1,2}.'); out=[];t=0; return
    end
    if nargout > 1,    t=t+toc; end

    if cRMSE
        if nargout > 1 && jndex==ITER,  tic;     end 
        u0est = ifft2((fft2(uztmp)).*S0); % <- to the object plane
        out = u0est(bourdery0,bourderx0); % the reconstructed object to be estimated
        if nargout > 1 && jndex==ITER,  t=t+toc; end
        figure(5)
        v2=abs(u0) - abs(out); RMSE(1,jndex)=sqrt(mean(mean( v2.^2 )));
        x2=angle(u0) - angle(out);tmp=mean(x2(:));x2 = wrap(x2-tmp); RMSE(2,jndex)=sqrt(mean(mean( x2.^2 )));
        
        subplot(2,3,1),imshow(abs(out),[]), title('amplitude, SBMIR'),xlabel(['RMSE=' num2str(RMSE(1,jndex))])
        subplot(2,3,2),plot(RMSE(1,1:jndex)), title('convergence: amplitude'),ylabel('RMSE'),xlabel('# of iterations');grid on 
        
        subplot(2,3,4),imshow(angle(out)+tmp,[]), title('phase, SBMIR'), xlabel(['RMSE=' num2str(RMSE(2,jndex))])
        subplot(2,3,5),plot(RMSE(2,1:jndex)), title('convergence: phase'),ylabel('RMSE'), xlabel(['K=' num2str(K) ',iter=' num2str(jndex) '/' num2str(ITER)]),grid on
        
        subplot(2,3,3),    plot(1:Nx0,abs(out(Ny0/2+1,:)),'b',1:Nx0,abs(u0(Ny0/2+1,:)),'r'),title(['amplitude: cross-section along the ' num2str(Ny0/2+1) '-th line'])
        axis([1,Nx0,0,max( max(abs(out(Ny0/2+1,:))), max(abs(u0(Ny0/2+1,:))) )])
        subplot(2,3,6),    plot(1:Nx0,angle(out(Ny0/2+1,:))+tmp,'b',1:Nx0,angle(u0(Ny0/2+1,:)),'r'),title(['phase: cross-section along the ' num2str(Ny0/2+1) '-th line'])
        axis([1,Nx0, min( min(angle(out(Ny0/2+1,:))+tmp), min(angle(u0(Ny0/2+1,:))) )    ,max( max(angle(out(Ny0/2+1,:))+tmp), max(angle(u0(Ny0/2+1,:))) )])
        
        drawnow
    end
    
end
fprintf('elapsed time = %8.4f seconds\n',t);
if ~cRMSE
    out = zeros(N,M,K);
    if op == 0 || op == 2
        for index=1:1:K-1
            out(:,:,index)=o(:,:,index).*ep(coordy,coordx);
            uztmp(coordy,coordx)=o(:,:,index).*ep(coordy,coordx);       
            uztmp=ifft2((fft2(uztmp)).*Sf); % ->          
            ph = atan2(imag(uztmp), real(uztmp));          ep = exp(i*ph);
        end
        out(:,:,K)=o(:,:,K).*ep(coordy,coordx);
    else
        for index=K:-1:2
            out(:,:,index)=o(:,:,index).*ep(coordy,coordx);
            uztmp(coordy,coordx)=o(:,:,index).*ep(coordy,coordx);       
            uztmp=ifft2((fft2(uztmp)).*Sf); % ->          
            ph = atan2(imag(uztmp), real(uztmp));          ep = exp(i*ph);
        end
        out(:,:,1)=o(:,:,1).*ep(coordy,coordx);
    end
end

function y=wrap(x)
% wrapping of the phase
y = atan2(sin(x),cos(x));    y(y==pi)=-pi; %  alternative > y = mod(x+pi,2*pi)-pi;

% 2010 - 2013 (c) TUT v. 2.0